function out = resynthesis(in, mask, fRange, winLength, winShift, fs)
% Produce an array of filtered responses from a Gammatone filterbank.
% The first variable is required. 
% numChan: number of filter channels.
% fRange: frequency range.
% fs: sampling frequency.
% Written by ZZ Jin, and adapted by DLW in Jan'07

if nargin < 3
    fRange = [80, 5000]; % default frequency range in Hz
end
if nargin < 4
    winLength = 320;    % default window length in sample points which is 20 ms for 16 KHz sampling frequency
end
if nargin < 5
    winShift = winLength/2;
end
if nargin < 6
    fs = 16000;     % default sampling frequency
end


filterOrder = 4;    % filter order
gL = 2048;          % gammatone filter length or 128 ms for 16 kHz sampling rate

sigLength = length(in);     % input signal length
[numChan,numFrame] = size(mask);     % number of channels and time frames

phase(1:numChan) = zeros(numChan,1);        % initial phases
erb_b = hz2erb(fRange);       % upper and lower bound of ERB
erb = [erb_b(1):diff(erb_b)/(numChan-1):erb_b(2)];     % ERB segment
cf = erb2hz(erb);       % center frequency array indexed by channel
b = 1.019*24.7*(4.37*cf/1000+1);       % rate of decay or bandwidth

increment = winLength/winShift;     % special treatment for beginning frames

coswin = zeros(winLength,1);
for i = 1:winLength     % calculate a raised cosine window
    coswin(i) = (1 + cos(2*pi*(i-1)/winLength - pi))/2;
end


% Generating gammatone impulse responses with middle-ear gain normalization

d = 2^ceil(log2(sigLength + gL));

% Generating gammatone impulse responses with middle-ear gain normalization
gL = 2048;      % gammatone filter length or 128 ms for 16 kHz sampling rate
gt = zeros(numChan,gL);
tmp_t = [1:gL]/fs;
for c = 1:numChan
    gain = 1/3*(2*pi*b(c)/fs).^4;
    gt(c,:) = gain*fs^3*tmp_t.^(filterOrder-1).*exp(-2*pi*b(c)*tmp_t).*cos(2*pi*cf(c)*tmp_t+phase(c));
end

% find scaling factor to make overall response in pass-band as close to 1
% as possible
y=zeros(d,1);
for c = 1:numChan
    x = fft(gt(c,:)',d);
    y = y + abs(x).^2;
end
fInds = round(cf(11:117)./(fs/d));
y = y(fInds);
z = ones(size(y));

scale = ((y'*z)/(y'*y))^.5;

gt = gt*scale;
out = zeros(size(in));
tmp_t = [1:gL]/fs;
for i = 1:numChan
    temp1 = real(ifft(fft(in,d).*fft(gt(i,:)',d)));
    temp2 = real(ifft(fft(flipud(temp1(1:sigLength)),d).*fft(gt(i,:)',d)));
    temp1 = flipud(temp2(1:sigLength));
    
    weight = zeros(sigLength,1);       % calculate weighting
    for m = 1:numFrame-increment/2+1      % mask value can be binary or rational           
        startpoint = (m-1)*winShift;
        if m <= increment/2                % shorter frame lengths for beginning frames or zero padding
            weight(1:startpoint+winLength/2) = weight(1:startpoint+winLength/2) + mask(i,m)*coswin(winLength/2-startpoint+1:end);
        else 
            weight(startpoint-winLength/2+1:startpoint+winLength/2) = weight(startpoint-winLength/2+1:startpoint+winLength/2) + mask(i,m)*coswin;
        end
    end
    
    out = out + temp1.*weight;
end
